import unittest
from mystix.language.shared.ast.Var import Var
from mystix.language.shared.ast.Value import Value
from mystix.language.shared.ast.math_ast.Operand import Operand
from mystix.language.shared.ast.math_ast.MathFuncs import MathFuncs
from mystix.language.shared.ast.math_ast.SimpleFunc import SimpleFunc
from mystix.language.shared.primitives import numerical, values
from mystix.language.evaluation.evaluator import Evaluator

class TestMathFuncs(unittest.TestCase):

    pass